		NAME	SPI_Test

; This version of the code is compatible only with the AT89C2051 due to the
; location of the data buffer and stack in RAM. The code may be modified to
; work with the AT89C1051 by relocating or resizing the buffer and stack to
; fit into the smaller amount of RAM available in the AT89C1051.
;
; Primitive functions SHOUT (SHift OUT) and SHIN (SHift IN) manage the
; movement of data on the serial bus. The serial clock is generated and
; timed by software. The code meets timing requirements when executed by
; an AT89Cx051 microcontroller with a 12 MHz clock. Code modifications may
; be required if a faster clock is substituted.
;
; The serial bus lines are mapped to microcontroller I/O port pins in the
; definitions of SCK, MISO and MOSI. Code to initialize the serial bus lines
; should be included in the user's application in the reset or initialization
; area.

SIZE		EQU	2000h		; bytes in code memory
FILL		EQU	0aah		; example fill value

; Device function codes.

WRITE_CODE	EQU	02h		; write code memory (Flash)
READ_CODE	EQU	01h		; read code memory (Flash)
WRITE_DATA	EQU	06h		; write data memory (EEPROM)
READ_DATA	EQU	05h		; read data memory (EEPROM)
DUMMY		EQU	55h		; dummy function data

; Register definitions.

index		EQU	r0		; buffer pointer
kount		EQU	r1		; byte count register
zdata		EQU	r1		; data register
addr_lo		EQU	r2		; 2-byte address register
addr_hi		EQU	r3		;

; Microcontroller connections to serial bus lines.

RST_		BIT	p1.7		; target control signal
SCK		BIT	p1.4		; serial clock
MOSI		BIT	p1.3		; serial data out
MISO		BIT	p1.2		; serial data in


		DSEG AT 20H

		ORG	60H		; stack origin
stack:		DS	20H		; stack depth


		CSEG

		ORG	0000H		; power on/reset vector
		jmp	on_reset

		ORG	0003H		; external interrupt 0 vector
		reti			; undefined

		ORG	000BH		; timer 0 overflow vector
		reti			; undefined

		ORG	0013H		; external interrupt 1 vector
		reti			; undefined

		ORG	001BH		; timer 1 overflow vector
		reti			; undefined

		ORG	0023H		; serial I/O interrupt vector
		reti			; undefined

		ORG	0080H		; begin code space
		USING	0		; register bank zero
on_reset:
		mov	sp, #(stack-1)	; initialize stack pointer

		; Initialize AT24Cxx serial bus lines.

		setb	MISO		; high
		setb	MOSI		; high
		clr	SCK		; low

		clr	RST_
		mov	a, #10
		call	delay_ms

		call	enable_write
		call	code_fill
		call	verify_code_fill
		jc	fault

		clr	p1.5		; turn on LED
		jmp	$

	fault:
		setb	p1.5		; turn off LED
		jmp	$


code_fill:

	; Fill every byte in code memory (Flash) with the same value.
	; Destroys A, DPTR, ADDR_HI:ADDR_LO.

		mov	dptr, #0	; initialize address pointer
	x51:
		mov	addr_lo, dpl	; set up address
		mov	addr_hi, dph	;
		mov	a, #FILL	; get fill data
		call	write_byte	; write data

		inc	dptr			; advance address pointer
;		mov	a, dpl			; check low byte
;		cjne	a, #(LOW SIZE), x51	; jump if not last
		mov	a, dph			; check high byte
		cjne	a, #(HIGH SIZE), x51	; jump if not last
		ret


verify_code_fill:

	; Verify that all bytes in code memory (Flash) match a fill value.
	; Returns CY set to indicate compare fail.
	; Destroys A, B, DPTR.

		mov	dptr, #0	; initialize address pointer/counter
	x83:
		mov	addr_lo, dpl	; set up address
		mov	addr_hi, dph	;
		call	read_byte	; read data
		cjne	a, #FILL, x86	; jump if compare error

		inc	dptr			; advance address pointer
		mov	a, dph			; check high byte
		cjne	a, #(HIGH SIZE), x83	; jump if not last
		clr	c			; clear error flag
		jmp	x87		; exit
	x86:
		setb	c		; set error flag
	x87:
		ret


erase_chip:

		mov	a, #0ach	; send first byte of erase code
		call	shout		;
		mov	a, #04h		; send second byte
		call	shout		;
		mov	a, #DUMMY	; send third byte
		call	shout		;

		mov	a, #10		; wait ten milliseconds
		call	delay_ms	;
		ret


enable_write:

		mov	a, #0ach	; send first byte of enable code
		call	shout		;
		mov	a, #53h		; send second byte
		call	shout		;
		mov	a, #DUMMY	; send third byte
		call	shout		;
		ret


write_byte:

	; Byte Write function.
	; Called with address in register pair ADDR_HI:ADDR_LO, data in A.
	; Waits for write cycle to complete.
	; Destroys A.

		push	acc		; save data

		mov	a, addr_hi	; high byte of address
		anl	a, #00011111b	; isolate five bits
		rl	a		; move five bits to top
		rl	a		;
		rl	a		;
		orl	a, #WRITE_CODE	; specify code write operation
		call	shout		; send it

		mov	a, addr_lo	; send low byte of address
		call	shout		;

		pop	acc		; get data
		call	shout		; send data

		mov	a, #3		; wait three milliseconds
		call	delay_ms
		ret


read_byte:

	; Byte Read function.
	; Called with address in register pair ADDR_HI:ADDR_LO.
	; Returns data in A.

		mov	a, addr_hi	; high byte of address
		anl	a, #00011111b	; isolate five bits
		rl	a		; move five bits to top
		rl	a		;
		rl	a		;
		orl	a, #READ_CODE	; specify code read operation
		call	shout		; send it

		mov	a, addr_lo	; send low byte of address
		call	shout		;

		call	shin		; read data
		ret


shout:

	; Shift out a byte, most significant bit first.
	; SCK expected low on entry. Return with SCK low.
	; Called with data to send in A.
	; Returns nothing.
	; Destroys A.

		push	b
		mov	b, #8		; bit counter
	x42:
		rlc	a		; move bit into CY		1
		mov	MOSI, c		; output bit			2
		nop			; enforce data setup		1
		nop			;				1
		setb	SCK		; raise clock			1
		nop			; enforce SCK high		1
		nop			;				1
		nop			;				1
		nop			;				1
		clr	SCK		; drop clock			1
		djnz	b, x42		; next bit			2

		pop	b
		ret


shin:

	; Shift in a byte, most significant bit first.
	; SCK expected low on entry. Return with SCK low.
	; Returns received data byte in A.

		push	b
		mov	b, #8		; bit counter
	x43:
		setb	SCK		; raise clock			1
		mov	c, MISO		; input bit			1
		rlc	a		; move bit into byte		1
		nop			; enforce SCK high		1
		nop			;				1
		clr	SCK		; drop clock			1
		nop			; enforce SCK low		1
		nop			;				1
		djnz	b, x43		; next bit			2

		pop	b
		ret


delay_ms:

	; Delay for one mS times the value in the accumulator.

		push	acc
		push	b

		mov	b, #0
	dd:
		djnz	b, $		; 500 uS @ 12 MHz
		djnz	b, $		; 500 uS @ 12 MHz
		djnz	acc, dd

		pop	b
		pop	acc
		ret


		END
